// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_FAILABLE_H
#define HU_VARADIISTVAN_SCPL_IO_FAILABLE_H
//------------------------------------------------------------------------------

#include "../types.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * An object whose instances can fail, and the reason of the failure
 * is indicated by an integral error code.
 */
class Failable
{
private:
    /**
     * The error code.
     */
    errorCode_t errorCode;

public:
    /**
     * Construct the object and initialize the error code to indicate
     * no error.
     */
    Failable();

    /**
     * Indicate if the object has failed.
     */
    bool failed() const;

    /**
     * Get the error code.
     */
    errorCode_t getErrorCode() const;

    /**
     * Discard the error code.
     */
    void repair();

protected:
    /**
     * Set the error code.
     */
    void setErrorCode(errorCode_t ec);
};

//------------------------------------------------------------------------------

/**
 * An object that references another failable object and fails
 * together with that object.
 *
 * The Child class should contain a function named getFailable()
 * which returns the failable object.
 */
template <class Child>
class FailableReference
{
public:
    /**
     * Indicate if the referenced object has failed.
     */
    bool failed() const;

    /**
     * Get the error code from the referenced object.
     */
    errorCode_t getErrorCode() const;

    /**
     * Discard the error code of the referenced object.
     */
    void repair();
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline Failable::Failable() :
    errorCode(XPL_NO_ERROR)
{
}

//------------------------------------------------------------------------------

inline bool Failable::failed() const
{
    return errorCode!=XPL_NO_ERROR;
}

//------------------------------------------------------------------------------

inline errorCode_t Failable::getErrorCode() const
{
    return errorCode;
}

//------------------------------------------------------------------------------

inline void Failable::repair()
{
    errorCode = XPL_NO_ERROR;
}

//------------------------------------------------------------------------------

inline void Failable::setErrorCode(errorCode_t ec)
{
    errorCode = ec;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

template <class Child>
inline bool FailableReference<Child>::failed() const
{
    return static_cast<const Child*>(this)->getFailable().failed();
}

//------------------------------------------------------------------------------

template <class Child>
inline errorCode_t FailableReference<Child>::getErrorCode() const
{
    return static_cast<const Child*>(this)->getFailable().getErrorCode();
}

//------------------------------------------------------------------------------

template <class Child>
inline void FailableReference<Child>::repair()
{
    static_cast<Child*>(this)->getFailable().repair();
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_FAILABLE_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
